<?php
/**
 * Blackbird Agency
 *
 * @category    Blackbird
 * @package     Blackbird
 * Date: 28/03/2024
 * Time: 10:30
 * @copyright   Copyright (c) 2024 Blackbird Agency. (http://black.bird.eu)
 * @author RÃ©gis Grutter (regis@bird.eu)
 */

declare(strict_types=1);

namespace Blackbird\HrefLang\Model\System;

use Magento\Framework\Data\OptionSourceInterface;
use Magento\Store\Model\StoreManagerInterface;

class Store extends \Magento\Framework\DataObject implements OptionSourceInterface
{
    protected array $_websiteCollection = [];

    protected array $_groupCollection = [];

    protected array $_storeCollection = [];

    protected StoreManagerInterface $_storeManager;

    private string $nonEscapableNbspChar;

    /**
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(StoreManagerInterface $storeManager)
    {
        $this->_storeManager = $storeManager;

        $this->nonEscapableNbspChar = html_entity_decode('&#160;', ENT_NOQUOTES, 'UTF-8');

        $this->reload();
    }

    /**
     * @return array
     */
    public function toOptionArray(): array
    {
        return $this->getStoreValuesForForm();
    }

    /**
     * Retrieve store values for form
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getStoreValuesForForm(): array
    {
        $options = [];

        foreach ($this->_websiteCollection as $website) {
            $websiteShow = false;
            foreach ($this->_groupCollection as $group) {
                if ($website->getId() !== $group->getWebsiteId()) {
                    continue;
                }
                $values = [];
                foreach ($this->_storeCollection as $store) {
                    if ($group->getId() !== $store->getGroupId()) {
                        continue;
                    }
                    if (!$websiteShow) {
                        $options[] = ['label' => $website->getName(), 'value' => []];
                        $websiteShow = true;
                    }
                    $values[] = [
                        'label' => str_repeat($this->nonEscapableNbspChar, 4) . $store->getName(),
                        'value' => $store->getId(),
                    ];
                }
                if (!empty($values)) {
                    $options[] = [
                        'label' => str_repeat($this->nonEscapableNbspChar, 4) . $group->getName(),
                        'value' => $values,
                    ];
                }
            }
        }
        array_walk(
            $options,
            function (&$item) {
                $item['__disableTmpl'] = true;
            }
        );
        return $options;
    }

    /**
     * Load/Reload Website collection
     * @return $this
     */
    protected function _loadWebsiteCollection(): static
    {
        $this->_websiteCollection = $this->_storeManager->getWebsites();
        return $this;
    }

    /**
     * Load/Reload Group collection
     * @return $this
     */
    protected function _loadGroupCollection(): static
    {
        $this->_groupCollection = [];
        foreach ($this->_storeManager->getWebsites() as $website) {
            foreach ($website->getGroups() as $group) {
                $this->_groupCollection[$group->getId()] = $group;
            }
        }
        return $this;
    }

    /**
     * Load/Reload Store collection
     * @return $this
     */
    protected function _loadStoreCollection(): static
    {
        $this->_storeCollection = $this->_storeManager->getStores();
        return $this;
    }

    /**
     * Load/Reload collection(s) by type
     * Allowed types: website, group, store or null for all
     * @param $type
     * @return $this
     */
    public function reload($type = null): static
    {
        if ($type === null) {
            $this->_loadWebsiteCollection();
            $this->_loadGroupCollection();
            $this->_loadStoreCollection();
        } else {
            switch ($type) {
                case \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE:
                    $this->_loadWebsiteCollection();
                    break;
                case \Magento\Store\Model\ScopeInterface::SCOPE_GROUP:
                    $this->_loadGroupCollection();
                    break;
                case \Magento\Store\Model\ScopeInterface::SCOPE_STORE:
                    $this->_loadStoreCollection();
                    break;
                default:
                    break;
            }
        }
        return $this;
    }
}
